green = Color.new(0, 255, 0)
red = Color.new(255, 0, 0)
message = "Welcome to the TrackBundler"
SaveDir = "ms0:/PSP/SAVEDATA"
DownDir = "ms0:/PSP"

DownSelect = 1
SaveSelect = 1

InputTimer = 0

done = false
OnDownSide = true -- this is our check for which side we're on
RenderHelp = false

function PrintToScreen()
	screen:print(0, 0, message, green)
	
	if System.powerIsPowerOnline() then
		battMsg = "Battery Status: On AC Power"
	else
		battMsg = "Battery Status: " .. System.powerGetBatteryLifePercent() .. "% remaining"
	end
	
	screen:print(480 - (string.len(battMsg) * 8), 0, battMsg, green)
	
end

function UnpackTrack( fileName )	

	screen:clear()
	screen:print(0, 0, "About to unpack track " .. fileName, green)
	screen:print(0, 10, "Press start to proceed, circle to cancel", green)
	screen.waitVblankStart()
	screen.flip()
	
	while true do
		pad = Controls.read()
		if pad:start() then
			break
		end
		
		if pad:circle() then
			return
		end
	end

	--Open input file
	fileInput = io.open( fileName, "rb" )

	-- get file size
	iTrackOffset = fileInput:seek("end")
	
	-- get folder name
	fileInput:seek("set", 85959 ) 
	TrackFolder = fileInput:read(16)
	
	-- get Track name
	fileInput:seek("set", 89319 ) 
	TrackName = fileInput:read(128)
	
	iTrackOffset = iTrackOffset - 89575
	
	fileInput:seek("set") -- reset to begining of file
	
	--extract data
	ICON0 = fileInput:read(23223)
	ICON1 = fileInput:read(61440)
	PARAM = fileInput:read(4912)
	TRACK = fileInput:read(iTrackOffset)

	-- close input file, as we dont need it anymore
	fileInput:close()
	
	-- write out files
	System.currentDirectory(SaveDir)	
	System.currentDirectory(TrackFolder)
	if System.currentDirectory() == SaveDir then
		System.makeDirectory(TrackFolder)
		System.currentDirectory(TrackFolder)
	else
	 -- we be overwriting here
	 -- Warn user 
		screen:clear()
		screen:print(0, 0, "About to overwrite a track " .. TrackName, green)
		screen:print(0, 10, "Press start to overwrite, circle to cancel", green)
		screen.waitVblankStart()
		screen.flip()
		
		while true do
			pad = Controls.read()
			if pad:start() then
				break
			end
			
			if pad:circle() then
				return
			end
		end
	end
	
	fileICON0 = io.open( "ICON0.PNG", "wb" )
	fileICON0:write(ICON0)
	fileICON0:flush()
	fileICON0:close()

	fileICON1 = io.open( "ICON1.PMF", "wb" )
	fileICON1:write(ICON1)
	fileICON1:flush()
	fileICON1:close()

	filePARAM = io.open( "PARAM.SFO", "wb" )
	filePARAM:write(PARAM)
	filePARAM:flush()
	filePARAM:close()

	fileTRACK = io.open( "TRACK.XDS", "wb" )
	fileTRACK:write(TRACK)
	fileTRACK:flush()
	fileTRACK:close()
	
	message = "Track " .. string.sub(TrackName,1,string.find(TrackName,"\000")-1) .. " successfully unpacked." 
end

function PackTrack() 

	-- Open Files
	fileICON0 = io.open( "ICON0.PNG", "rb" )
	fileICON1 = io.open( "ICON1.PMF", "rb" )
	filePARAM = io.open( "PARAM.SFO", "rb" )
	fileTRACK = io.open( "TRACK.XDS", "rb" )
	
	-- Get Track Name
	filePARAM:seek("set", 4656)
	TrackName = filePARAM:read( 127 )
	filePARAM:seek("set")
	
	newFilename = string.format( "%s.GST", string.sub(TrackName,1,string.find(TrackName,"\000")) ) 
	
	-- Warn user 
	screen:clear()
	screen:print(0, 0, "About to pack track " .. string.sub(TrackName,1,string.find(TrackName,"\000")), green)
	screen:print(0, 10, "to " .. newFilename, green)
	screen:print(0, 20, "Press start to proceed, circle to cancel", green)
	screen.waitVblankStart()
	screen.flip()
	
	while true do
		pad = Controls.read()
		if pad:start() then
			break
		end
		
		if pad:circle() then
			return
		end
	end
	
	-- extract data
	ICON0 = fileICON0:read( "*all" )
	ICON1 = fileICON1:read( "*all" )
	PARAM = filePARAM:read( "*all" )
	TRACK = fileTRACK:read( "*all" )
	
	-- Open output file, should check to see if the file exsists
	System.currentDirectory(DownDir)
	
	OverWrite = io.open( newFilename, "rb" )
	
	if OverWrite then
	-- close the file
		OverWrite:close()
	-- Warn user 
		screen:clear()
		screen:print(0, 0, "About to overwrite packed track " .. newFilename, green)
		screen:print(0, 10, "Press start to overwrite, circle to cancel", green)
		screen.waitVblankStart()
		screen.flip()
		
		while true do
			pad = Controls.read()
			if pad:start() then
				break
			end
			
			if pad:circle() then
				return
			end
		end
	end
	
	fileOut = io.open( newFilename, "wb" )
	
	if not fileOut then
		message = "Error writing out file " .. newFilename
		return
	end

	-- write to output file
	fileOut:write( ICON0 )
	fileOut:write( ICON1 )
	fileOut:write( PARAM )
	fileOut:write( TRACK )

	-- close files and flush buffers
	fileOut:flush()
	fileOut:close()

	fileICON0:close()
	fileICON1:close()
	filePARAM:close()
	fileTRACK:close()
	
	message = "Track " .. string.sub(TrackName,1,string.find(TrackName,"\000")-1) .. " successfully packed." 
end

function RefreshDown() -- Refreshes the folder contents of the downloads directory
	finishedDown = false

	DownContents = System.listDirectory(DownDir)

	while not finishedDown do -- filter out unwanted files
		for idx,file in DownContents do
			if file.name == "." then
				table.remove(DownContents , idx)
				break
			end
			
			if file.directory == false and string.find( file.name, ".GST" ) == nil and string.find( file.name, ".gst" ) == nil then
				table.remove(DownContents , idx)
				break
			end
			
			if idx == table.getn(DownContents) then
				finishedDown = true
			end
		end
	end

	DownSelect = 1
end

function RefreshSave() -- Refreshes the folder contents of the save directory
	finishedSave = false

	SaveContents = System.listDirectory(SaveDir)

	while not finishedSave do -- filter out unwanted files
		for idx,file in SaveContents do
			if file.name == "." then
				table.remove(SaveContents , idx)
				break
			end
			if file.directory == true and string.find( file.name, "TRACK" ) == nil then
				table.remove(SaveContents , idx)
				break
			end
			if idx >= table.getn(SaveContents) then
				finishedSave = true
			end
		end
		if table.getn(SaveContents) == 0 then
			finishedSave = true
		end
	end
	
	finishedSave = false

	if table.getn(SaveContents) == 0 then return end

	SaveTracknames = {} -- empty table to store Tracknames
	
	-- now that we have only track folders, lets get the track names
	-- goto Save dir
	System.currentDirectory( SaveDir )
	while not finishedSave do
		for idx,file in SaveContents do
			-- goto track dir
			System.currentDirectory( SaveContents[idx].name )
			
			-- open PARAM.SF0
			fileRead = io.open( "PARAM.SFO", "rb" )
			
			if not fileRead then
				message = "Cannot open file PARAM.SFO in dir " .. file.name
				return
			end
			
			-- goto offset 0x00001230
			fileRead:seek("set", 4656)
			
	
			-- read in 128 bytes
			TrackName = fileRead:read( 127 )
			fileRead:close()
			
			-- trim off null terminator
			newTrackname = string.sub(TrackName,1,string.find(TrackName,"\000")-1)
			
			-- add to SaveTracknames table
			SaveTracknames[idx] = newTrackname

			System.currentDirectory( ".." )
			
			if idx == table.getn(SaveContents) then
				finishedSave = true
			end
		end
	end

	SaveSelect = 1
end

function RenderDown() -- Renders the left side (downloads)
	for i = 1, table.getn(DownContents) do
		if i == DownSelect and OnDownSide == true then
			screen:print(25,i*10+25,"> " .. DownContents[i].name .. " <",red)
		else
			screen:print(25,i*10+25,DownContents[i].name,green)
		end
	end
end

function RenderSave() -- Renders the right side (saves)

	if table.getn(SaveContents) == 0 then return end 

	for i = 1, table.getn(SaveTracknames) do
		if i == SaveSelect and OnDownSide == false then
			screen:print(265,i*10+25,"> " .. string.sub(SaveContents[i].name, -2) .. ": " .. SaveTracknames[i] .. " <",red)
		else
			screen:print(265,i*10+25,string.sub(SaveContents[i].name, -2) .. ": " .. SaveTracknames[i],green)
		end
	end
end

function RenderHelpScreen()
	screen:print(0, 10, "Keys:", green)
	
	screen:print(0, 20, "Up       : Move up listing", green)
	screen:print(0, 30, "Down     : Move down listing", green)
	screen:print(0, 40, "Left     : Change to Track listing", green)
	screen:print(0, 50, "Right    : Change to Save listing", green)
	
	screen:print(0, 70, "Cross    : Select item", green)
	screen:print(0, 80, "Triangle : Show / Hide help", green)
	
	screen:print(0, 100, "Start    : Quit", green)
	
	screen:print(0, 120, "The left pane will only show folders and .gst files. This is", green)
	screen:print(0, 130, "the side that will unpack the track file into your save" , green)
	screen:print(0, 140, "folder." , green)
	
	screen:print(0, 160, "The right pane is your SAVEDATA folder. This is where your ", green)
	screen:print(0, 170, "saved games and tracks are. You can package up tracks for" , green)
	screen:print(0, 180, "downloading here." , green)

	screen:print(0, 200, "GripShift is the property of Sidhe Interactive. I in no way", green)
	screen:print(0, 210, "claim rights on anything they have done. This app is my work" , green)
	screen:print(0, 220, "and wish to get credit where is is due." , green)
end

function Render() 
	screen:clear()
	
	if RenderHelp == true then	
		RenderHelpScreen()
	else
		screen:blit(0,0,BGImage)
		
		RenderDown()
		RenderSave()
	end
	
	PrintToScreen()
	
	screen.waitVblankStart()
	screen:flip()
end

function ExecSelect()
	-- check which side we're on
	if OnDownSide == true then
		if DownContents[DownSelect].directory == true then
			System.currentDirectory(DownDir)	
			System.currentDirectory(DownContents[DownSelect].name)
			DownDir = System.currentDirectory()
			message = DownDir -- this can be removed later
			RefreshDown()
		else
			-- Unpack file
			System.currentDirectory(DownDir)	
			UnpackTrack(DownContents[DownSelect].name)
			RefreshSave()
		end
	else
		if SaveContents[SaveSelect].directory == true then
			-- Pack folder into .gst file
			System.currentDirectory(SaveDir)	
			System.currentDirectory(SaveContents[SaveSelect].name)
			PackTrack()
			RefreshDown()
		end
	end
end

-- Deletes a file
function DeleteFile( fileName )
	if fileName == nil then
		return
	end
	
	screen:clear()
	screen:print(0, 0, "About to delete file " .. fileName, green)
	screen:print(0, 10, "Press start to delete, circle to cancel", green)
	screen.waitVblankStart()
	screen.flip()
	
	while true do
		pad = Controls.read()
		if pad:start() then
			break
		end
		
		if pad:circle() then
			return
		end
	end
	
	System.removeFile( fileName )
end

function GetInput()
	pad = Controls.read()
	if pad:start() then
		done = true
	end

	if pad:select() then
		screen:save("screen.png")
	end

	if pad:cross() then
		ExecSelect()
	end

	if pad:circle() then
		if OnDownSide == true then
			file = DownDir .. "/" .. DownContents[DownSelect].name
			DeleteFile(file)
			RefreshDown()
		end
	end

	if pad:square() then
		
	end

	if pad:triangle() then
		RenderHelp = not RenderHelp
	end

	if pad:up() then
		if OnDownSide == true then
			DownSelect = DownSelect - 1
			if DownSelect < 1 then
				DownSelect = 1
			end
		else
			SaveSelect = SaveSelect - 1
			if SaveSelect < 1 then
				SaveSelect = 1
			end
		end
	end

	if pad:down() then
		if OnDownSide == true then
			DownSelect = DownSelect + 1
			if DownSelect > table.getn(DownContents) then
				DownSelect = table.getn(DownContents)
			end
		else
			SaveSelect = SaveSelect + 1
			if SaveSelect > table.getn(SaveContents) then
				SaveSelect = table.getn(SaveContents)
			end
		end
	end
	
	if pad:left() then
		OnDownSide = true
	end
	
	if pad:right() then
		OnDownSide = false
	end
end

--main loop
screen:clear()
screen.flip()
screen:clear()

-- Load background image
BGImage = Image.load( "BG.jpg" )

-- Get folder listings
RefreshDown()
RefreshSave()

System.currentDirectory(DownDir)

while done == false do
	Render()
	
	if InputTimer >= 4 then
		GetInput()
		InputTimer = 0
	else
		InputTimer = InputTimer + 1
	end
end

System.currentDirectory(appsDir)